#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

int main(int argc, char **argv) {

    // Make sure we have exactly one argument, the name of the file
    if (argc != 2) {
        fprintf(stderr, "Usage: %s [filename]\n", argv[0]);
        return 1;
    }

    // Open the file with fopen. "r" means we want to read the contents.
    FILE *f = fopen(argv[1], "r");

    // If f is null, we failed to open the file, so we print an error and exit.
    if (f == NULL) {
        perror("Failed to open file");
        return 1;
    }

    // 'seek' to the end of the file, then...
    // ftell tells us what position we're at in the file.
    // since we're at the end, this is the file's size.
    fseek(f, 0, SEEK_END);
    size_t file_size = ftell(f);
    printf("File size: %lu\n", file_size);

    // Allocate enough memory to store the whole file
    // then seek back to the start and read it into that memory
    char *data = malloc(file_size + 1);
    fseek(f, 0, SEEK_SET);
    fread(data, 1, file_size, f);

    // we're done with the file, so we need to fclose it.
    fclose(f);

    // Add an ending NULL byte to the read file data, to make sure
    // it prints as a string nicely.
    data[file_size] = '\0';

    // Print the file's contents
    printf("%s\n", data);

    // Then free that allocated memory! and we're done!
    free(data);
    return 0;

}
